<?php
// ************************************************************
// Very simple to-do list
// John Watson
// john {at} flagrantdisregard {dotcom}
// ************************************************************
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// '
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ************************************************************
// $Id$
// ************************************************************
//
// ToeKnee: added 'done' command to strikeout an entry
// and cleanup stuff
// tlacey {at} bupswee {dotnet}
// 2006.09.27

if (!defined('PmWiki')) exit();

/*
This recipe works by creating input forms that know how to modify the content
of the current page by adding or removing a single item.  Multiple forms on
a single page are supported by identifying each form with a unique name

	(:todo-add-<uniquename> <above|below> <symbol>:)
	
When items are added to the page, they are either added above or below the
form and are prefixed with the symbols given in the last argument.  For example:

	(:todo-add-list1 above *:)
	
creates an input form that adds new lines above it prefixed with *, creating
a bulletted list entry form.  The <uniquename> can only contain letters and
numbers.

When an item is added using the form, markup is added as a suffix

	(:todo-delete-<id>:)
	
that is rendered as a link.  When clicked, the link will remove the item from
the page.  The <id> is generated as an md5 hash of the line being added but
can be anything as long as it is unique and contains only letters nad numbers.
This allows one to add the (:todo-delete-<id>:) markup to existing lines by
hand if desired.

To use this recipe, add the following lines to your local/config.php:

// Very simple todo lists
//$TodoDeleteText = "delete";
//$TodoDoneText = "done";
//$TodoUndoText = "undo";
//$TodoAddText = "New item:";
//$TodoCleanText = "Clean List:";
include_once("$FarmD/cookbook/lesssimpletodo.php");

$TodoDeleteText is the string used as the text of the delete link.
$TodoAddText is the label placed in front of the input box for new items.
$TodoDoneText is the string used as the text of the done link.

Additionally, you can style the components using the classes "todoform"
for the input form and "tododelete" for the delete link.  For example:

	.todoform {
		color: #999;
		font-size: 80%;
		margin: 0;
		padding: 0;
	}
	
	.todoform input {
		font-weight: normal;
		border: 0;
		border-bottom: 1px solid #eee;
		width: 400px;
	}
	a.todoclean {
		color: #999 !important;
		font-size: 80%;
		text-decoration: none !important;
	}

	a.tododelete {
		color: #999 !important;
		text-transform: lowercase;
		font-size: 70%;
		text-decoration: none !important;
	}

	a.tododone {
		color: #999 !important;
		text-transform: lowercase;
		font-size: 70%;
		text-decoration: none !important;
	}

	a.todoundo {
		color: #999 !important;
		text-transform: lowercase;
		font-size: 70%;
		text-decoration: none !important;
	}
	
	a:hover.tododelete {
		color: #999 !important;
		text-transform: lowercase;
		font-size: 70%;
		text-decoration: underline !important;
	}
*/

Markup("todo", "block", "/\(:todo-add-(\w+?) (above|below) (.+?):\)/e",
	"MakeTodoForm('$1', \$pagename)");

Markup("todo-done", "block", "/\(:todo-done-(.*?):\)/e",
	"MakeTodoLink('delete', '$1', \$pagename).' '.MakeTodoLink('done', '$1', \$pagename)"
);

Markup("todo-undo", "block", "/\(:todo-undo-(.*?):\)/e",
	"MakeTodoLink('delete', '$1', \$pagename).' '.MakeTodoLink('undo', '$1', \$pagename)"
);

$HandleActions['todo-add'] = 'HandleAddTodo';
$HandleActions['todo-clean'] = 'HandleCleanList';
$HandleActions['todo-delete'] = 'HandleDeleteTodo';
$HandleActions['todo-done'] = 'HandleDoneTodo';
$HandleActions['todo-undo'] = 'HandleUndoTodo';

function TodoRedirect($pagename, $urlfmt='$PageUrl') {
	global $TodoRedirect;
	if(function_exists($TodoRedirect)) {
		$TodoRedirect($pagename);
	} else {
		Redirect($pagename, $urlfmt);
	}
}

// Make Todo add form
function MakeTodoForm($listname, $pn) {
	global $ScriptUrl, $TodoAddText, $TodoCleanText;

	$a = doCreateQnotesPage($listname, $pn); if($a) return $a;

	return  "<table valign=center><tr><td>"
		."<form id='todoform$listname' class='todoform' action='$ScriptUrl' method='post'>"
		.(isset($TodoAddText)?$TodoAddText:'::')." "
		."<input type='hidden' name='todoform' value='$listname' />"
		."<input type='hidden' name='n' value='$pn' />"
		."<input type='hidden' name='action' value='todo-add' />"
		."<input type='text' id='tododata' name='tododata' />"
		."<input type='submit' id='submit' name='submit' value='Add' />"
		."<script type='text/javascript'>document.getElementById('tododata').focus();</script>"
		."</form>"
		."</td><td>"
		."<a class='todoclean' href='$ScriptUrl?n=$pn&todoitem=$listname&action=todo-clean'>".(isset($TodoCleanText)?$TodoCleanText:'Clean List')."</a>"
		."</td></tr></table>";
}

// Make Delete/Done/Undo link
function MakeTodoLink($type, $hash, $pn) {
	global $ScriptUrl, $TodoDeleteText, $TodoDoneText, $TodoUndoText;
	switch ($type) {
		case 'delete' :
			return "<a class='tododelete' href='$ScriptUrl?n=$pn" . 
			"&todoitem=$hash&action=todo-delete'>" .
			(isset($TodoDeleteText)?$TodoDeleteText:'Delete')."</a>";
		break;
		case 'done' :
			return "<a class='tododone' href='$ScriptUrl?n=$pn" .
			"&todoitem=$hash&action=todo-done'>" . 
			(isset($TodoDoneText)?$TodoDoneText:'Done')."</a>";
		break;
		case 'undo' :
			return "<a class='todoundo' href='$ScriptUrl?n=$pn" .
			"&todoitem=$hash&action=todo-undo'>" . 
			(isset($TodoUndoText)?$TodoUndoText:'Undo')."</a>";
		break;
	}
}

// Add an item to a page
function HandleAddTodo() {
	global $pagename;
	$now = getdate(time());
	if (isset($_REQUEST['tododata'])) {
		$todoform = $_REQUEST['todoform'];
 		$tododata = $_REQUEST['tododata'];
 		$tododata = trim(stripmagic($tododata));

		$data = TodoMakeHash($todoform, $tododata, $_REQUEST['datestamp']);

		// Escape dollar signs in the data or you end up with weird preg_replace() issues
		$tododata = preg_replace('/\$/', '\\\$', $tododata);
		$page = RetrieveAuthPage($pagename, 'edit', true);
		$donetag = '(:todo-done-'.$data['hs'].':)';
		if (!$page) Abort('?cannot read $pagename');
		$new = $page;
		if (preg_match("/\(:todo-add-$todoform above (.+?):\)/", $page['text'])) {
			$new['text'] = preg_replace("/\(:todo-add-$todoform (above|below) (.+?):\)/", "$2 $tododata $donetag\n(:todo-add-$todoform $1 $2:)", $page['text'], 1);
		} else if (preg_match("/\(:todo-add-$todoform below (.+?):\)/", $page['text'])) {
			$new['text'] = preg_replace("/\(:todo-add-$todoform (above|below) (.+?):\)/", "(:todo-add-$todoform $1 $2:)\n$2 $tododata $donetag", $page['text'], 1);
		}
		PostPage($pagename, $page, $new);

		doAddQnote($todoform, $tododata, $data);
		TodoRedirect($pagename);
	}
}

function doCreateQnotesPage($listname, $pn) {
	global $QNotes, $ROEPatterns, $ROSPatterns, $EnablePost;
	$listname = ucfirst($listname); $qnp = 'Todo.'.$listname;
	if( ! PageExists($qnp)) {
		$QNotes['isDiff'] = 0;
		//$EnablePost = 1;
		$ROSPatterns ["/{{List}}/"] = $listname;
		$ROSPatterns ["/{{behaviour}}/"] = 'card';
		CreateQnotesPage($qnp, 'Templates.Todo', 2);
	}
	$text = RetrieveAuthSection('Todo/'.$listname, '#selflink');
	$a = MarkupToHTML($pn, $text);
	if($qnp !==  str_replace('/', '.', $pn)) { return $a; } return false;
}
function doAddQnote($todoform, $tododata, $data) {
	$qnp = 'Todo.'.ucfirst($todoform);
	$qn = '/'.$data['d'].'/'.$data['t'].$data['s'].$data['h'];
	AddQnote($qnp, $qn, $tododata, '', 'update', 1);
}

function TodoMakeHash($listname, $str, $ds='') {
	$listname = trim(stripmagic($listname));
	$now = getdate(time()); $data = array(); $len = 12;
	$ds = ($ds) ? substr(trim($ds), 0, $len) : '';
	$ds = (strlen($ds) == $len) ? $ds : strftime('%Y%m%d%H%M', $now[0]);
	$data['d'] = substr($ds, 0, 8); $data['t'] = substr($ds, 8, 4);
	$data['s'] = ''; $data['h'] = md5($listname.$str); $data['hs'] = $ds.$data['h'].'-'.$listname;
	return $data;
}
function TodoHashFmt($hashstr, $esc=true) {
        list($hash, $listname) = explode('-', $hashstr);
	$hash = strtolower(trim($hash));
	$hash = stripmagic($hash);
	$hash = preg_replace('/[^\\dabcdef]/', '', $hash);
	$listname = preg_replace('/[^\\w]/', '', $listname);
	if(strlen($hash) == 44) { return $hash.'-'.$listname; }
	return false;
}
function doDeleteQnote($todoitem, $list) {
	$qnp = 'Todo.'.ucfirst($list);
	$qn = '/'.substr($todoitem, 0, 8).'/'.substr($todoitem, 8, 36);
	DeleteQnote($qnp, $qn, 'delete', 1);
}

// Remove an item from a page
function HandleDeleteTodo() {
	global $pagename;
	
	if (isset($_REQUEST['todoitem'])) {
		$todoitem = TodoHashFmt($_REQUEST['todoitem']);
		list($hash, $listname) = explode('-', $todoitem);
		$page = RetrieveAuthPage($pagename, 'edit', true);
		if (!$page) Abort('?cannot read $pagename');
		$new = $page;
		$new['text'] = preg_replace("/^.*?\(:todo-(done|undo)-$todoitem:\).*?\n/m", "", $page['text'], 1);
		PostPage($pagename, $page, $new);

		doDeleteQnote($todoitem, $listname);
		TodoRedirect($pagename);
	}
}


//rewrite done tasks
function DoneCallback($match) {
	$newtext = str_replace("(:todo-done-", "-} (:todo-undo-", $match[0]);
	$pieces = explode(" ", $newtext);
	$pieces[1] = '{-'.$pieces[1];
	$newtext = implode(' ', $pieces);
	$newtext = preg_replace("/\\s+?\\-\\}/si", "-}", $newtext);
	return $newtext;
}


// Strikeout done tasks
function HandleDoneTodo() {
	global $pagename;

	if (isset($_REQUEST['todoitem'])) {
		$todoitem = TodoHashFmt($_REQUEST['todoitem']);
		$page = RetrieveAuthPage($pagename, 'edit', true);
		if (!$page) Abort('?cannot read $pagename');
		$new = $page;
		$new['text'] = preg_replace_callback("/^.*?\(:todo-done-$todoitem:\).*?\n/m", "DoneCallback", $page['text'], 1);
		PostPage($pagename, $page, $new);
		TodoRedirect($pagename);
	}
}


//rewrite undo tasks
function UndoCallback($match) {
	$newtext = str_replace(" {-", " ", $match[0]);
	$newtext = str_replace("-} ", " ", $newtext);
	$newtext = str_replace("(:todo-undo-", "(:todo-done-", $newtext);
	return $newtext;
}


// Strikeout done tasks
function HandleUndoTodo() {
	global $pagename;

	if (isset($_REQUEST['todoitem'])) {
		$todoitem = TodoHashFmt($_REQUEST['todoitem']);
		$page = RetrieveAuthPage($pagename, 'edit', true);
		if (!$page) Abort('?cannot read $pagename');
		$new = $page;
		$new['text'] = preg_replace_callback("/^.*?\(:todo-undo-$todoitem:\).*?\n/m", "UndoCallback", $page['text'], 1);
		PostPage($pagename, $page, $new);
		TodoRedirect($pagename);
	}
}

// remove done tasks
function HandleCleanList() {
	global $pagename;

	if (isset($_REQUEST['todoitem'])) {
 		$todoitem = $_REQUEST['todoitem'];
 		$todoitem = stripmagic($todoitem);
		$page = RetrieveAuthPage($pagename, 'edit', true);
		if (!$page) Abort('?cannot read $pagename');
		$new = $page;
		$new['text'] = preg_replace("/^.*?\(:todo-undo-(.*?)-$todoitem:\)\n/m", "", $page['text'], -1);
		PostPage($pagename, $page, $new);

		preg_match_all("/^.*?\(:todo-undo-(.*?-$todoitem):\)\n/m", $page['text'], $m);
		if($m[1] && count($m[1]) > 0) {
			foreach($m[1] as $item) { doDeleteQnote($item, $todoitem); }
		}

		TodoRedirect($pagename);
	}
}



